@@ -14,7 +14,7 @@ import qualified Canonizer as C
 import Condition (parseConditionThrows)
 import Control.Exception (Exception (displayException), SomeException, handle, throw, throwIO)
 import Control.Exception.Base
-import Control.Monad (unless, when, (>=>))
+import Control.Monad (forM_, unless, when, (>=>))
 import Data.Char (toLower, toUpper)
 import Data.Foldable (for_)
 import qualified Data.Foldable
@@ -50,13 +50,15 @@ import Yaml (normalizationRules)
 import qualified Yaml as Y
 
 data PrintProgramContext = PrintProgCtx
-  { sugar :: SugarType,
-    line :: LineFormat,
-    xmirCtx :: XmirContext,
-    nonumber :: Bool,
-    expression :: Maybe String,
-    label :: Maybe String,
-    outputFormat :: IOFormat
+  { sugar :: SugarType
+  , line :: LineFormat
+  , xmirCtx :: XmirContext
+  , nonumber :: Bool
+  , compress :: Bool
+  , expression :: Maybe String
+  , label :: Maybe String
+  , meetPrefix :: Maybe String
+  , outputFormat :: IOFormat
   }
 
 data CmdException
@@ -66,8 +68,8 @@ data CmdException
   deriving (Exception)
 
 instance Show CmdException where
-  show InvalidCLIArguments {..} = printf "Invalid set of arguments: %s" message
-  show CouldNotReadFromStdin {..} = printf "Could not read input from stdin\nReason: %s" message
+  show InvalidCLIArguments{..} = printf "Invalid set of arguments: %s" message
+  show CouldNotReadFromStdin{..} = printf "Could not read input from stdin\nReason: %s" message
   show CouldNotDataize = "Could not dataize given program"
 
 data Command
@@ -86,91 +88,95 @@ instance Show IOFormat where
   show LATEX = "latex"
 
 data OptsDataize = OptsDataize
-  { logLevel :: LogLevel,
-    logLines :: Integer,
-    inputFormat :: IOFormat,
-    outputFormat :: IOFormat,
-    sugarType :: SugarType,
-    flat :: LineFormat,
-    omitListing :: Bool,
-    omitComments :: Bool,
-    nonumber :: Bool,
-    sequence :: Bool,
-    depthSensitive :: Bool,
-    quiet :: Bool,
-    maxDepth :: Integer,
-    maxCycles :: Integer,
-    hide :: [String],
-    show' :: [String],
-    expression :: Maybe String,
-    label :: Maybe String,
-    stepsDir :: Maybe FilePath,
-    inputFile :: Maybe FilePath
+  { logLevel :: LogLevel
+  , logLines :: Int
+  , inputFormat :: IOFormat
+  , outputFormat :: IOFormat
+  , sugarType :: SugarType
+  , flat :: LineFormat
+  , omitListing :: Bool
+  , omitComments :: Bool
+  , nonumber :: Bool
+  , sequence :: Bool
+  , depthSensitive :: Bool
+  , quiet :: Bool
+  , compress :: Bool
+  , maxDepth :: Int
+  , maxCycles :: Int
+  , hide :: [String]
+  , show' :: [String]
+  , expression :: Maybe String
+  , label :: Maybe String
+  , meetPrefix :: Maybe String
+  , stepsDir :: Maybe FilePath
+  , inputFile :: Maybe FilePath
   }
 
 data OptsExplain = OptsExplain
-  { logLevel :: LogLevel,
-    logLines :: Integer,
-    rules :: [FilePath],
-    normalize :: Bool,
-    shuffle :: Bool,
-    targetFile :: Maybe FilePath
+  { logLevel :: LogLevel
+  , logLines :: Int
+  , rules :: [FilePath]
+  , normalize :: Bool
+  , shuffle :: Bool
+  , targetFile :: Maybe FilePath
   }
 
 data OptsRewrite = OptsRewrite
-  { logLevel :: LogLevel,
-    logLines :: Integer,
-    inputFormat :: IOFormat,
-    outputFormat :: IOFormat,
-    sugarType :: SugarType,
-    flat :: LineFormat,
-    must :: Must,
-    normalize :: Bool,
-    shuffle :: Bool,
-    omitListing :: Bool,
-    omitComments :: Bool,
-    depthSensitive :: Bool,
-    nonumber :: Bool,
-    inPlace :: Bool,
-    sequence :: Bool,
-    canonize :: Bool,
-    maxDepth :: Integer,
-    maxCycles :: Integer,
-    rules :: [FilePath],
-    hide :: [String],
-    show' :: [String],
-    expression :: Maybe String,
-    label :: Maybe String,
-    targetFile :: Maybe FilePath,
-    stepsDir :: Maybe FilePath,
-    inputFile :: Maybe FilePath
+  { logLevel :: LogLevel
+  , logLines :: Int
+  , inputFormat :: IOFormat
+  , outputFormat :: IOFormat
+  , sugarType :: SugarType
+  , flat :: LineFormat
+  , must :: Must
+  , normalize :: Bool
+  , shuffle :: Bool
+  , omitListing :: Bool
+  , omitComments :: Bool
+  , depthSensitive :: Bool
+  , nonumber :: Bool
+  , inPlace :: Bool
+  , sequence :: Bool
+  , canonize :: Bool
+  , compress :: Bool
+  , maxDepth :: Int
+  , maxCycles :: Int
+  , rules :: [FilePath]
+  , hide :: [String]
+  , show' :: [String]
+  , expression :: Maybe String
+  , label :: Maybe String
+  , meetPrefix :: Maybe String
+  , targetFile :: Maybe FilePath
+  , stepsDir :: Maybe FilePath
+  , inputFile :: Maybe FilePath
   }
 
 data OptsMerge = OptsMerge
-  { logLevel :: LogLevel,
-    logLines :: Integer,
-    inputFormat :: IOFormat,
-    outputFormat :: IOFormat,
-    sugarType :: SugarType,
-    flat :: LineFormat,
-    omitListing :: Bool,
-    omitComments :: Bool,
-    targetFile :: Maybe FilePath,
-    inputs :: [FilePath]
+  { logLevel :: LogLevel
+  , logLines :: Int
+  , inputFormat :: IOFormat
+  , outputFormat :: IOFormat
+  , sugarType :: SugarType
+  , flat :: LineFormat
+  , omitListing :: Bool
+  , omitComments :: Bool
+  , targetFile :: Maybe FilePath
+  , inputs :: [FilePath]
   }
 
 data OptsMatch = OptsMatch
-  { logLevel :: LogLevel,
-    logLines :: Integer,
-    sugarType :: SugarType,
-    flat :: LineFormat,
-    pattern :: Maybe String,
-    when' :: Maybe String,
-    inputFile :: Maybe FilePath
+  { logLevel :: LogLevel
+  , logLines :: Int
+  , sugarType :: SugarType
+  , flat :: LineFormat
+  , pattern :: Maybe String
+  , when' :: Maybe String
+  , inputFile :: Maybe FilePath
   }
 
-validateIntegerOption :: (Integer -> Bool) -> String -> Integer -> ReadM Integer
-validateIntegerOption cmp msg num
+validateIntOption :: (Int -> Bool) -> String -> Int -> ReadM Int
+validateIntOption cmp msg num
   | cmp num = return num
   | otherwise = readerError msg
 
@@ -196,10 +202,10 @@ optLogLevel =
       "NONE" -> Right NONE
       _ -> Left $ "unknown log-level: " <> lvl
 
-optLogLines :: Parser Integer
+optLogLines :: Parser Int
 optLogLines =
   option
-    (auto >>= validateIntegerOption (>= -1) "--log-lines must be >= -1")
+    (auto >>= validateIntOption (>= -1) "--log-lines must be >= -1")
     (long "log-lines" <> metavar "LINES" <> help "Amount of lines printed to console per each log operation (0 - print nothing, -1 - no limits)" <> value 25 <> showDefault)
 
 optRule :: Parser [FilePath]
@@ -224,16 +230,16 @@ optOutputFormat =
 argInputFile :: Parser (Maybe FilePath)
 argInputFile = optional (argument str (metavar "FILE" <> help "Path to input file"))
 
-optMaxDepth :: Parser Integer
+optMaxDepth :: Parser Int
 optMaxDepth =
   option
-    (auto >>= validateIntegerOption (> 0) "--max-depth must be positive")
+    (auto >>= validateIntOption (> 0) "--max-depth must be positive")
     (long "max-depth" <> metavar "DEPTH" <> help "Maximum number of rewriting iterations per rule" <> value 25 <> showDefault)
 
-optMaxCycles :: Parser Integer
+optMaxCycles :: Parser Int
 optMaxCycles =
   option
-    (auto >>= validateIntegerOption (> 0) "--max-cycles must be positive")
+    (auto >>= validateIntOption (> 0) "--max-cycles must be positive")
     (long "max-cycles" <> metavar "CYCLES" <> help "Maximum number of rewriting cycles across all rules" <> value 25 <> showDefault)
 
 optDepthSensitive :: Parser Bool
@@ -251,6 +257,9 @@ optExpression = optional (strOption (long "expression" <> metavar "NAME" <> help
 optLabel :: Parser (Maybe String)
 optLabel = optional (strOption (long "label" <> metavar "NAME" <> help "Name for 'label' element when rendering to LaTeX (see --output option)"))
 
+optMeetPrefix :: Parser (Maybe String)
+optMeetPrefix = optional (strOption (long "meet-prefix" <> metavar "PREFIX" <> help "Prefix to be inserted before index in \\phiMeet{} and \\phiAgain{} LaTeX functions, e.g. \\phiMeet{foo:1}"))
+
 optHide :: Parser [String]
 optHide = many (strOption (long "hide" <> metavar "FQN" <> help "Location of object to exclude from result and intermediate programs after rewriting. Must be a valid dispatch expression; e.g. Q.org.eolang"))
 
@@ -296,6 +305,9 @@ optOmitListing = switch (long "omit-listing" <> help "Omit full program listing
 optOmitComments :: Parser Bool
 optOmitComments = switch (long "omit-comments" <> help "Omit comments in XMIR output")
 
+optCompress :: Parser Bool
+optCompress = switch (long "compress" <> help "Compress expressions in LaTeX output using \\phiMeet{} and \\phiAgain{} functions")
+
 _intermediateOptions :: String
 _intermediateOptions = intercalate ", " ["--sequence", "--steps-dir"]
 
@@ -327,12 +339,14 @@ dataizeParser =
             <*> optSequence
             <*> optDepthSensitive
             <*> switch (long "quiet" <> help "Don't print the result of dataization")
+            <*> optCompress
             <*> optMaxDepth
             <*> optMaxCycles
             <*> optHide
             <*> optShow
             <*> optExpression
             <*> optLabel
+            <*> optMeetPrefix
             <*> optStepsDir
             <*> argInputFile
         )
@@ -364,6 +378,7 @@ rewriteParser =
             <*> switch (long "in-place" <> help "Edit file in-place instead of printing to output")
             <*> optSequence
             <*> switch (long "canonize" <> help "Rename all functions attached to λ binding with F1, F2, etc.")
+            <*> optCompress
             <*> optMaxDepth
             <*> optMaxCycles
             <*> optRule
@@ -371,6 +386,7 @@ rewriteParser =
             <*> optShow
             <*> optExpression
             <*> optLabel
+            <*> optMeetPrefix
             <*> optTarget
             <*> optStepsDir
             <*> argInputFile
@@ -431,11 +447,11 @@ handler e = case fromException e of
 setLogger :: Command -> IO ()
 setLogger cmd =
   let (level, lines) = case cmd of
-        CmdRewrite OptsRewrite {logLevel, logLines} -> (logLevel, logLines)
-        CmdDataize OptsDataize {logLevel, logLines} -> (logLevel, logLines)
-        CmdExplain OptsExplain {logLevel, logLines} -> (logLevel, logLines)
-        CmdMerge OptsMerge {logLevel, logLines} -> (logLevel, logLines)
-        CmdMatch OptsMatch {logLevel, logLines} -> (logLevel, logLines)
+        CmdRewrite OptsRewrite{logLevel, logLines} -> (logLevel, logLines)
+        CmdDataize OptsDataize{logLevel, logLines} -> (logLevel, logLines)
+        CmdExplain OptsExplain{logLevel, logLines} -> (logLevel, logLines)
+        CmdMerge OptsMerge{logLevel, logLines} -> (logLevel, logLines)
+        CmdMatch OptsMatch{logLevel, logLines} -> (logLevel, logLines)
    in setLogConfig level lines
 
 invalidCLIArguments :: String -> IO a
@@ -446,7 +462,7 @@ runCLI args = handle handler $ do
   cmd <- handleParseResult (execParserPure defaultPrefs parserInfo args)
   setLogger cmd
   case cmd of
-    CmdRewrite OptsRewrite {..} -> do
+    CmdRewrite OptsRewrite{..} -> do
       validateOpts
       excluded <- expressionsToFilter "hide" hide
       included <- expressionsToFilter "show" show'
@@ -456,7 +472,7 @@ runCLI args = handle handler $ do
       program <- parseProgram input inputFormat
       let listing = if null rules' then const input else (\prog -> P.printProgram' prog (sugarType, UNICODE, flat))
           xmirCtx = XmirContext omitListing omitComments listing
-          printCtx = PrintProgCtx sugarType flat xmirCtx nonumber expression label outputFormat
+          printCtx = PrintProgCtx sugarType flat xmirCtx nonumber compress expression label meetPrefix outputFormat
           _canonize = if canonize then C.canonize else id
           _hide = (`F.exclude` excluded)
           _show = (`F.include` included)
@@ -475,9 +491,14 @@ runCLI args = handle handler $ do
             (inPlace && isJust targetFile)
             (invalidCLIArguments "The options --in-place and --target cannot be used together")
           when (length show' > 1) (invalidCLIArguments "The option --show can be used only once")
-          validateLatexOptions outputFormat nonumber expression label
+          validateLatexOptions
+            outputFormat
+            [(nonumber, "nonumber"), (compress, "compress")]
+            [(expression, "expression"), (label, "label"), (meetPrefix, "meet-prefix")]
           validateMust' must
-          validateXmirOptions outputFormat omitListing omitComments
+          validateXmirOptions
+            outputFormat
+            [(omitListing, "omit-listing"), (omitComments, "omit-comments")]
         output :: Maybe FilePath -> String -> IO ()
         output target prog = case (inPlace, target, inputFile) of
           (True, _, Just file) -> do
@@ -500,13 +521,13 @@ runCLI args = handle handler $ do
             buildTerm
             must
             (saveStepFunc stepsDir ctx)
-    CmdDataize OptsDataize {..} -> do
+    CmdDataize OptsDataize{..} -> do
       validateOpts
       excluded <- expressionsToFilter "hide" hide
       included <- expressionsToFilter "show" show'
       input <- readInput inputFile
       prog <- parseProgram input inputFormat
-      let printCtx = PrintProgCtx sugarType flat defaultXmirContext nonumber expression label outputFormat
+      let printCtx = PrintProgCtx sugarType flat defaultXmirContext nonumber compress expression label meetPrefix outputFormat
           _hide = (`F.exclude` excluded)
           _show = (`F.include` included)
       (maybeBytes, seq) <- dataize prog (context prog printCtx)
@@ -515,8 +536,13 @@ runCLI args = handle handler $ do
       where
         validateOpts :: IO ()
         validateOpts = do
-          validateLatexOptions outputFormat nonumber expression label
-          validateXmirOptions outputFormat omitListing omitComments
+          validateLatexOptions
+            outputFormat
+            [(nonumber, "nonumber"), (compress, "compress")]
+            [(expression, "expression"), (label, "label"), (meetPrefix, "meet-prefix")]
+          validateXmirOptions
+            outputFormat
+            [(omitListing, "omit-listing"), (omitComments, "omit-comments")]
           when (length show' > 1) (invalidCLIArguments "The option --show can be used only once")
         context :: Program -> PrintProgramContext -> DataizeContext
         context program ctx =
@@ -527,7 +553,7 @@ runCLI args = handle handler $ do
             depthSensitive
             buildTerm
             (saveStepFunc stepsDir ctx)
-    CmdExplain OptsExplain {..} -> do
+    CmdExplain OptsExplain{..} -> do
       validateOpts
       rules' <- getRules normalize shuffle rules
       let latex = explainRules rules'
@@ -538,14 +564,14 @@ runCLI args = handle handler $ do
           when
             (null rules && not normalize)
             (throwIO (InvalidCLIArguments "Either --rule or --normalize must be specified"))
-    CmdMerge OptsMerge {..} -> do
+    CmdMerge OptsMerge{..} -> do
       validateOpts
       inputs' <- traverse (readInput . Just) inputs
       progs <- traverse (`parseProgram` inputFormat) inputs'
       prog <- merge progs
       let listing = const (P.printProgram' prog (sugarType, UNICODE, flat))
           xmirCtx = XmirContext omitListing omitComments listing
-          printProgCtx = PrintProgCtx sugarType flat xmirCtx False Nothing Nothing outputFormat
+          printProgCtx = PrintProgCtx sugarType flat xmirCtx False False Nothing Nothing Nothing outputFormat
       prog' <- printProgram printProgCtx prog
       output targetFile prog'
       where
@@ -554,8 +580,8 @@ runCLI args = handle handler $ do
           when
             (null inputs)
             (throwIO (InvalidCLIArguments "At least one input file must be specified for 'merge' command"))
-          validateXmirOptions outputFormat omitListing omitComments
-    CmdMatch OptsMatch {..} -> do
+          validateXmirOptions outputFormat [(omitListing, "omit-listing"), (omitComments, "omit-comments")]
+    CmdMatch OptsMatch{..} -> do
       input <- readInput inputFile
       prog <- parseProgram input PHI
       if isNothing pattern
@@ -573,7 +599,7 @@ runCLI args = handle handler $ do
 
 -- Prepare saveStepFunc
 saveStepFunc :: Maybe FilePath -> PrintProgramContext -> SaveStepFunc
-saveStepFunc stepsDir ctx@PrintProgCtx {..} = saveStep stepsDir ioToExt (printProgram ctx)
+saveStepFunc stepsDir ctx@PrintProgCtx{..} = saveStep stepsDir ioToExt (printProgram ctx)
   where
     ioToExt :: String
     ioToExt
@@ -599,31 +625,29 @@ expressionsToFilter opt = traverse (parseExpressionThrows >=> asFilter)
             )
 
 -- Validate LaTeX options
-validateLatexOptions :: IOFormat -> Bool -> Maybe String -> Maybe String -> IO ()
-validateLatexOptions outputFormat nonumber expression label = do
-  when
-    (nonumber && outputFormat /= LATEX)
-    (invalidCLIArguments "The --nonumber option can stay together with --output=latex only")
-  when
-    (isJust expression && outputFormat /= LATEX)
-    (invalidCLIArguments "The --expression option can stay together with --output=latex only")
-  when
-    (isJust label && outputFormat /= LATEX)
-    (invalidCLIArguments "The --label option can stay together with --output=latex only")
+validateLatexOptions :: IOFormat -> [(Bool, String)] -> [(Maybe String, String)] -> IO ()
+validateLatexOptions LATEX _ _ = pure ()
+validateLatexOptions _ bools maybes = do
+  let (bools', opts) = unzip bools
+      msg = "The --%s option can stay together with --output=latex only"
+  validateBoolOpts (zip bools' (map (printf msg) opts))
+  forM_
+    maybes
+    (\(maybe', opt) -> when (isJust maybe') (invalidCLIArguments (printf msg opt)))
 
 -- Validate 'must' option
 validateMust' :: Must -> IO ()
 validateMust' must = for_ (validateMust must) invalidCLIArguments
 
 -- Validate options for output to XMIR
-validateXmirOptions :: IOFormat -> Bool -> Bool -> IO ()
-validateXmirOptions outputFormat omitListing omitComments = do
-  when
-    (outputFormat /= XMIR && omitListing)
-    (invalidCLIArguments "--omit-listing can be used only with --output=xmir")
-  when
-    (outputFormat /= XMIR && omitComments)
-    (invalidCLIArguments "--omit-comments can be used only with --output=xmir")
+validateXmirOptions :: IOFormat -> [(Bool, String)] -> IO ()
+validateXmirOptions XMIR _ = pure ()
+validateXmirOptions _ bools =
+  let (bools', opts) = unzip bools
+   in validateBoolOpts (zip bools' (map (printf "The --%s can be used only with --output=xmir") opts))
+
+validateBoolOpts :: [(Bool, String)] -> IO ()
+validateBoolOpts bools = forM_ bools (\(bool, msg) -> when bool (invalidCLIArguments msg))
 
 -- Read input from file or stdin
 readInput :: Maybe FilePath -> IO String
@@ -643,16 +667,16 @@ parseProgram xmir XMIR = do
   xmirToPhi doc
 
 printRewrittens :: PrintProgramContext -> [Rewritten] -> IO String
-printRewrittens ctx@PrintProgCtx {..} rewrittens
-  | outputFormat == LATEX = pure (rewrittensToLatex rewrittens (LatexContext sugar line nonumber expression label))
+printRewrittens ctx@PrintProgCtx{..} rewrittens
+  | outputFormat == LATEX = pure (rewrittensToLatex rewrittens (LatexContext sugar line nonumber compress expression label meetPrefix))
   | otherwise = mapM (printProgram ctx . fst) rewrittens <&> intercalate "\n"
 
 -- Convert program to corresponding String format
 printProgram :: PrintProgramContext -> Program -> IO String
-printProgram PrintProgCtx {..} prog = case outputFormat of
+printProgram PrintProgCtx{..} prog = case outputFormat of
   PHI -> pure (P.printProgram' prog (sugar, UNICODE, line))
   XMIR -> programToXMIR prog xmirCtx <&> printXMIR
-  LATEX -> pure (programToLaTeX prog (LatexContext sugar line nonumber expression label))
+  LATEX -> pure (programToLaTeX prog (LatexContext sugar line nonumber compress expression label meetPrefix))
 
 -- Get rules for rewriting depending on provided flags
 getRules :: Bool -> Bool -> [FilePath] -> IO [Y.Rule]
