@@ -9,28 +9,28 @@
 
 -- This module provides commonly used helper functions for other modules
 module Misc
-  ( numToBts,
-    strToBts,
-    bytesToBts,
-    btsToStr,
-    btsToNum,
-    withVoidRho,
-    allPathsIn,
-    ensuredFile,
-    shuffle,
-    toDouble,
-    btsToUnescapedStr,
-    attributesFromBindings,
-    attributesFromBindings',
-    attributeFromBinding,
-    uniqueBindings,
-    uniqueBindings',
-    validateYamlObject,
-    matchDataObject,
-    pattern DataObject,
-    pattern DataString,
-    pattern DataNumber,
-    pattern BaseObject,
+  ( numToBts
+  , strToBts
+  , bytesToBts
+  , btsToStr
+  , btsToNum
+  , withVoidRho
+  , allPathsIn
+  , ensuredFile
+  , shuffle
+  , toDouble
+  , btsToUnescapedStr
+  , attributesFromBindings
+  , attributesFromBindings'
+  , attributeFromBinding
+  , uniqueBindings
+  , uniqueBindings'
+  , validateYamlObject
+  , matchDataObject
+  , pattern DataObject
+  , pattern DataString
+  , pattern DataNumber
+  , pattern BaseObject
   )
 where
 
@@ -68,8 +68,8 @@ data FsException
   deriving (Exception)
 
 instance Show FsException where
-  show FileDoesNotExist {..} = printf "File '%s' does not exist" file
-  show DirectoryDoesNotExist {..} = printf "Directory '%s' does not exist" dir
+  show FileDoesNotExist{..} = printf "File '%s' does not exist" file
+  show DirectoryDoesNotExist{..} = printf "Directory '%s' does not exist" dir
 
 matchBaseObject :: Expression -> Maybe String
 matchBaseObject (ExDispatch (ExDispatch (ExDispatch ExGlobal (AtLabel "org")) (AtLabel "eolang")) (AtLabel label)) = Just label
@@ -208,7 +208,7 @@ allPathsIn dir = do
 
 -- >>> toDouble 5
 -- 5.0
-toDouble :: Integer -> Double
+toDouble :: Int -> Double
 toDouble = fromIntegral
 
 -- >>> btsToWord8 BtEmpty
@@ -245,7 +245,7 @@ word8ToBytes bts = BtMany (map (printf "%02X") bts)
 -- Left 42
 -- >>> btsToNum (BtMany ["40", "45"])
 -- Expected 8 bytes for conversion, got 2
-btsToNum :: Bytes -> Either Integer Double
+btsToNum :: Bytes -> Either Int Double
 btsToNum hx =
   let bytes = btsToWord8 hx
    in if length bytes /= 8
