@@ -17,6 +17,7 @@ import Paths_phino (version)
 import System.Directory (doesDirectoryExist, doesFileExist, listDirectory, removeDirectoryRecursive, removeFile)
 import System.Exit (ExitCode (ExitFailure))
 import System.IO
+import System.Info (os)
 import Test.Hspec
 import Text.Printf (printf)
 
@@ -99,6 +100,9 @@ resource file = "test-resources/cli/" <> file
 rule :: String -> String
 rule file = "--rule=" <> resource file
 
+isWindows :: Bool
+isWindows = os == "mingw32"
+
 spec :: Spec
 spec = do
   it "prints version" $
@@ -168,10 +172,10 @@ spec = do
       it "with wrong attribute and valid error message" $
         testCLIFailed
           ["rewrite", resource "with-$this-attribute.phi"]
-          [ "[ERROR]: Couldn't parse given phi program, cause: program:10:13:",
-            "10 |             $this ↦ ⟦⟧",
-            "   |             ^^",
-            "unexpected \"$t\""
+          [ "[ERROR]: Couldn't parse given phi program, cause: program:10:13:"
+          , "10 |             $this ↦ ⟦⟧"
+          , "   |             ^^"
+          , "unexpected \"$t\""
           ]
 
       it "with --output != latex and --nonumber" $
@@ -204,6 +208,18 @@ spec = do
             ["rewrite", "--label=foo", "--output=phi"]
             ["--label option can stay together with --output=latex only"]
 
+      it "with --compress and --output != latex" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--compress", "--output=phi"]
+            ["--compress option can stay together with --output=latex only"]
+
+      it "with --meet-prefix and --output != latex" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--meet-prefix=foo", "--output=phi"]
+            ["--meet-prefix option can stay together with --output=latex only"]
+
       it "with wrong --hide option" $
         withStdin "{[[]]}" $
           testCLIFailed
@@ -215,7 +231,7 @@ spec = do
           testCLIFailed
             ["rewrite", "--show=Q.x.y", "--show=hello"]
             ["The option --show can be used only once"]
-      
+
       it "with wrong --show option" $
         withStdin "{[[]]}" $
           testCLIFailed
@@ -254,15 +270,15 @@ spec = do
       testCLISucceeded
         ["rewrite", "--normalize", resource "normalize.phi"]
         [ unlines
-            [ "Φ ↦ ⟦",
-              "  x ↦ ⟦",
-              "    ρ ↦ ⟦",
-              "      y ↦ ⟦ ρ ↦ ∅ ⟧,",
-              "      ρ ↦ ∅",
-              "    ⟧",
-              "  ⟧,",
-              "  ρ ↦ ∅",
-              "⟧"
+            [ "Φ ↦ ⟦"
+            , "  x ↦ ⟦"
+            , "    ρ ↦ ⟦"
+            , "      y ↦ ⟦ ρ ↦ ∅ ⟧,"
+            , "      ρ ↦ ∅"
+            , "    ⟧"
+            , "  ⟧,"
+            , "  ρ ↦ ∅"
+            , "⟧"
             ]
         ]
 
@@ -271,13 +287,13 @@ spec = do
         testCLISucceeded
           ["rewrite", "--normalize"]
           [ unlines
-              [ "Φ ↦ ⟦",
-                "  a ↦ ⟦",
-                "    b ↦ ⟦ ρ ↦ ∅ ⟧,",
-                "    ρ ↦ ∅",
-                "  ⟧,",
-                "  ρ ↦ ∅",
-                "⟧"
+              [ "Φ ↦ ⟦"
+              , "  a ↦ ⟦"
+              , "    b ↦ ⟦ ρ ↦ ∅ ⟧,"
+              , "    ρ ↦ ∅"
+              , "  ⟧,"
+              , "  ρ ↦ ∅"
+              , "⟧"
               ]
           ]
 
@@ -297,44 +313,44 @@ spec = do
       withStdin "Q -> [[ x_o -> QQ.z(y -> 5), q$ -> T, w -> $, ^ -> Q, @ -> 1, y -> \"H$@^M\", L> Fu_nc ]]" $
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet"]
-          [ "\\begin{phiquation}",
-            "\\Big\\{[[\n",
-            "  |x\\char95{}o| -> QQ.|z|( |y| -> 5 ),\n",
-            "  |q\\char36{}| -> T,\n",
-            "  |w| -> $,\n",
-            "  ^ -> Q,\n",
-            "  @ -> 1,\n",
-            "  |y| -> \"H$@^M\",\n",
-            "  L> |Fu\\char95{}nc|\n",
-            "]]\\Big\\}",
-            "\\end{phiquation}"
+          [ "\\begin{phiquation}"
+          , "\\Big\\{[[\n"
+          , "  |x\\char95{}o| -> QQ.|z|( |y| -> 5 ),\n"
+          , "  |q\\char36{}| -> T,\n"
+          , "  |w| -> $,\n"
+          , "  ^ -> Q,\n"
+          , "  @ -> 1,\n"
+          , "  |y| -> \"H$@^M\",\n"
+          , "  L> |Fu\\char95{}nc|\n"
+          , "]]\\Big\\}"
+          , "\\end{phiquation}"
           ]
 
     it "rewrites as LaTeX without numeration" $
       withStdin "Q -> [[ x -> 5 ]]" $
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet", "--nonumber", "--flat"]
-          [ "\\begin{phiquation*}",
-            "\\Big\\{[[ |x| -> 5 ]]\\Big\\}",
-            "\\end{phiquation*}"
+          [ "\\begin{phiquation*}"
+          , "\\Big\\{[[ |x| -> 5 ]]\\Big\\}"
+          , "\\end{phiquation*}"
           ]
 
     it "rewrite as LaTeX with expression name" $
       withStdin "Q -> [[ x -> 5 ]]" $
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet", "--flat", "--expression=foo"]
-          [ "\\begin{phiquation}",
-            "\\phiExpression{foo} \\Big\\{[[ |x| -> 5 ]]\\Big\\}.\n",
-            "\\end{phiquation}"
+          [ "\\begin{phiquation}"
+          , "\\phiExpression{foo} \\Big\\{[[ |x| -> 5 ]]\\Big\\}.\n"
+          , "\\end{phiquation}"
           ]
 
     it "rewrite as LaTeX with label name" $
       withStdin "Q -> [[ x -> 5 ]]" $
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet", "--flat", "--label=foo"]
-          [ "\\begin{phiquation}\n\\label{foo}\n",
-            "\\Big\\{[[ |x| -> 5 ]]\\Big\\}.\n",
-            "\\end{phiquation}"
+          [ "\\begin{phiquation}\n\\label{foo}\n"
+          , "\\Big\\{[[ |x| -> 5 ]]\\Big\\}.\n"
+          , "\\end{phiquation}"
           ]
 
     it "rewrites with XMIR as input" $
@@ -342,11 +358,11 @@ spec = do
         testCLISucceeded
           ["rewrite", "--input=xmir", "--sweet"]
           [ unlines
-              [ "{⟦",
-                "  app ↦ ⟦",
-                "    x ↦ Φ.number",
-                "  ⟧",
-                "⟧}"
+              [ "{⟦"
+              , "  app ↦ ⟦"
+              , "    x ↦ Φ.number"
+              , "  ⟧"
+              , "⟧}"
               ]
           ]
 
@@ -365,41 +381,73 @@ spec = do
     it "prints many programs with --sequence" $
       withStdin "{[[ x -> \"foo\" ]]}" $
         testCLISucceeded
-          [ "rewrite",
-            rule "first.yaml",
-            rule "second.yaml",
-            "--max-depth=1",
-            "--max-cycles=2",
-            "--sequence",
-            "--sweet",
-            "--flat"
+          [ "rewrite"
+          , rule "first.yaml"
+          , rule "second.yaml"
+          , "--max-depth=1"
+          , "--max-cycles=2"
+          , "--sequence"
+          , "--sweet"
+          , "--flat"
           ]
           [ unlines
-              [ "{⟦ x ↦ \"foo\" ⟧}",
-                "{Φ.x( y ↦ \"foo\" )}",
-                "{⟦ x ↦ \"foo\" ⟧}"
+              [ "{⟦ x ↦ \"foo\" ⟧}"
+              , "{Φ.x( y ↦ \"foo\" )}"
+              , "{⟦ x ↦ \"foo\" ⟧}"
               ]
           ]
 
     it "prints only one latex preamble with --sequence" $
       withStdin "{[[ x -> \"foo\" ]]}" $
         testCLISucceeded
-          [ "rewrite",
-            rule "first.yaml",
-            rule "second.yaml",
-            "--max-depth=1",
-            "--max-cycles=2",
-            "--sequence",
-            "--sweet",
-            "--flat",
-            "--output=latex"
+          [ "rewrite"
+          , rule "first.yaml"
+          , rule "second.yaml"
+          , "--max-depth=1"
+          , "--max-cycles=2"
+          , "--sequence"
+          , "--sweet"
+          , "--flat"
+          , "--output=latex"
           ]
           [ unlines
-              [ "\\begin{phiquation}",
-                "\\Big\\{[[ |x| -> \"foo\" ]]\\Big\\} \\leadsto_{\\nameref{r:first}}",
-                "  \\leadsto \\Big\\{Q.|x|( |y| -> \"foo\" )\\Big\\} \\leadsto_{\\nameref{r:second}}",
-                "  \\leadsto \\Big\\{[[ |x| -> \"foo\" ]]\\Big\\}.",
-                "\\end{phiquation}"
+              [ "\\begin{phiquation}"
+              , "\\Big\\{[[ |x| -> \"foo\" ]]\\Big\\} \\leadsto_{\\nameref{r:first}}"
+              , "  \\leadsto \\Big\\{Q.|x|( |y| -> \"foo\" )\\Big\\} \\leadsto_{\\nameref{r:second}}"
+              , "  \\leadsto \\Big\\{[[ |x| -> \"foo\" ]]\\Big\\}."
+              , "\\end{phiquation}"
+              ]
+          ]
+
+    it "prints meet prefix with --meet-prefix=foo in LaTeX" $
+      withStdin "{[[ x -> ?, y -> $.x ]](x -> [[ D> 42- ]]).y}" $
+        testCLISucceeded
+          ["rewrite", "--normalize", "--sweet", "--sequence", "--output=latex", "--flat", "--compress", "--meet-prefix=foo"]
+          [ unlines
+              [ "\\begin{phiquation}"
+              , "\\Big\\{[[ |x| -> ?, |y| -> |x| ]]( |x| -> [[ D> 42- ]] ).|y|\\Big\\} \\leadsto_{\\nameref{r:copy}}"
+              , "  \\leadsto \\Big\\{\\phiMeet{foo:1}{[[ |x| -> [[ D> 42- ]], |y| -> |x| ]]}.|y|\\Big\\} \\leadsto_{\\nameref{r:dot}}"
+              , "  \\leadsto \\Big\\{\\phiAgain{foo:1}.|x|( ^ -> \\phiAgain{foo:1} )\\Big\\} \\leadsto_{\\nameref{r:dot}}"
+              , "  \\leadsto \\Big\\{[[ D> 42- ]]( ^ -> \\phiAgain{foo:1}, ^ -> \\phiAgain{foo:1} )\\Big\\} \\leadsto_{\\nameref{r:copy}}"
+              , "  \\leadsto \\Big\\{[[ D> 42-, ^ -> \\phiAgain{foo:1} ]]( ^ -> \\phiAgain{foo:1} )\\Big\\} \\leadsto_{\\nameref{r:stay}}"
+              , "  \\leadsto \\Big\\{[[ D> 42-, ^ -> \\phiAgain{foo:1} ]]\\Big\\}."
+              , "\\end{phiquation}"
+              ]
+          ]
+
+    it "prints with compressed expressions in LaTeX" $
+      withStdin "{[[ x -> ?, y -> $.x ]](x -> [[ D> 42- ]]).y}" $
+        testCLISucceeded
+          ["rewrite", "--normalize", "--sweet", "--sequence", "--output=latex", "--flat", "--compress"]
+          [ unlines
+              [ "\\begin{phiquation}"
+              , "\\Big\\{[[ |x| -> ?, |y| -> |x| ]]( |x| -> [[ D> 42- ]] ).|y|\\Big\\} \\leadsto_{\\nameref{r:copy}}"
+              , "  \\leadsto \\Big\\{\\phiMeet{1}{[[ |x| -> [[ D> 42- ]], |y| -> |x| ]]}.|y|\\Big\\} \\leadsto_{\\nameref{r:dot}}"
+              , "  \\leadsto \\Big\\{\\phiAgain{1}.|x|( ^ -> \\phiAgain{1} )\\Big\\} \\leadsto_{\\nameref{r:dot}}"
+              , "  \\leadsto \\Big\\{[[ D> 42- ]]( ^ -> \\phiAgain{1}, ^ -> \\phiAgain{1} )\\Big\\} \\leadsto_{\\nameref{r:copy}}"
+              , "  \\leadsto \\Big\\{[[ D> 42-, ^ -> \\phiAgain{1} ]]( ^ -> \\phiAgain{1} )\\Big\\} \\leadsto_{\\nameref{r:stay}}"
+              , "  \\leadsto \\Big\\{[[ D> 42-, ^ -> \\phiAgain{1} ]]\\Big\\}."
+              , "\\end{phiquation}"
               ]
           ]
 
@@ -496,9 +544,9 @@ spec = do
         testCLISucceeded
           ["rewrite", "--sweet", rule "infinite.yaml", "--max-depth=1", "--max-cycles=2"]
           [ unlines
-              [ "{⟦",
-                "  x ↦ \"x_hi_hi\"",
-                "⟧}"
+              [ "{⟦"
+              , "  x ↦ \"x_hi_hi\""
+              , "⟧}"
               ]
           ]
 
@@ -529,10 +577,10 @@ spec = do
     it "removes unnecessary rho bindings in primitive applications" $
       withStdin
         ( unlines
-            [ "{[[",
-              "  z -> [[ x -> [[ t -> 42 ]].t ]].x,",
-              "  org -> [[ eolang -> [[ bytes -> [[ data -> ? ]], number -> [[ as-bytes -> ? ]] ]] ]]",
-              "]]}"
+            [ "{[["
+            , "  z -> [[ x -> [[ t -> 42 ]].t ]].x,"
+            , "  org -> [[ eolang -> [[ bytes -> [[ data -> ? ]], number -> [[ as-bytes -> ? ]] ]] ]]"
+            , "]]}"
             ]
         )
         ( testCLISucceeded
@@ -546,11 +594,11 @@ spec = do
           ["rewrite", "--log-level=debug", "--log-lines=4", "--normalize"]
           [ intercalate
               "\n"
-              [ "[DEBUG]: Applied 'COPY' (28 nodes -> 25 nodes)",
-                "{⟦",
-                "  x ↦ ⟦",
-                "    y ↦ 5",
-                "---| log is limited by --log-lines=4 option |---"
+              [ "[DEBUG]: Applied 'COPY' (28 nodes -> 25 nodes)"
+              , "{⟦"
+              , "  x ↦ ⟦"
+              , "    y ↦ 5"
+              , "---| log is limited by --log-lines=4 option |---"
               ]
           ]
 
@@ -575,15 +623,15 @@ spec = do
           ["dataize", "--sequence", "--output=latex", "--flat", "--sweet"]
           [ intercalate
               "\n"
-              [ "\\begin{phiquation}",
-                "\\Big\\{[[ @ -> [[ |x| -> [[ D> 01-, |y| -> ? ]]( |y| -> [[]] ) ]].|x| ]]\\Big\\} \\leadsto_{\\nameref{r:contextualize}}",
-                "  \\leadsto \\Big\\{[[ |x| -> [[ D> 01-, |y| -> ? ]]( |y| -> [[]] ) ]].|x|\\Big\\} \\leadsto_{\\nameref{r:copy}}",
-                "  \\leadsto \\Big\\{[[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]].|x|\\Big\\} \\leadsto_{\\nameref{r:dot}}",
-                "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]] ]]( ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] )\\Big\\} \\leadsto_{\\nameref{r:copy}}",
-                "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]], ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] ]]\\Big\\} \\leadsto_{\\nameref{r:Mprim}}",
-                "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]], ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] ]]\\Big\\}.",
-                "\\end{phiquation}",
-                "01-"
+              [ "\\begin{phiquation}"
+              , "\\Big\\{[[ @ -> [[ |x| -> [[ D> 01-, |y| -> ? ]]( |y| -> [[]] ) ]].|x| ]]\\Big\\} \\leadsto_{\\nameref{r:contextualize}}"
+              , "  \\leadsto \\Big\\{[[ |x| -> [[ D> 01-, |y| -> ? ]]( |y| -> [[]] ) ]].|x|\\Big\\} \\leadsto_{\\nameref{r:copy}}"
+              , "  \\leadsto \\Big\\{[[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]].|x|\\Big\\} \\leadsto_{\\nameref{r:dot}}"
+              , "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]] ]]( ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] )\\Big\\} \\leadsto_{\\nameref{r:copy}}"
+              , "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]], ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] ]]\\Big\\} \\leadsto_{\\nameref{r:Mprim}}"
+              , "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]], ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] ]]\\Big\\}."
+              , "\\end{phiquation}"
+              , "01-"
               ]
           ]
 
@@ -650,18 +698,21 @@ spec = do
         ["Either --rule or --normalize must be specified"]
 
     it "writes to target file" $
-      bracket
-        (openTempFile "." "explainXXXXXX.tex")
-        (\(path, _) -> removeFile path)
-        ( \(path, h) -> do
-            hClose h
-            testCLISucceeded
-              ["explain", "--normalize", printf "--target=%s" path]
-              [printf "was saved in '%s'" path]
-            content <- readFile path
-            content `shouldContain` "\\documentclass{article}"
-            content `shouldContain` "\\begin{document}"
-        )
+      if isWindows
+        then pendingWith "Skipped on Windows due to file locking issues"
+        else
+          bracket
+            (openTempFile "." "explainXXXXXX.tex")
+            (\(path, _) -> removeFile path)
+            ( \(path, h) -> do
+                hClose h
+                testCLISucceeded
+                  ["explain", "--normalize", printf "--target=%s" path]
+                  [printf "was saved in '%s'" path]
+                content <- readFile path
+                content `shouldContain` "\\documentclass{article}"
+                content `shouldContain` "\\begin{document}"
+            )
 
   describe "merge" $ do
     it "merges single program" $
@@ -673,17 +724,17 @@ spec = do
       testCLISucceeded
         ["merge", "--sweet", resource "number.phi", resource "bytes.phi", resource "string.phi"]
         [ unlines
-            [ "{⟦",
-              "  org ↦ ⟦",
-              "    eolang ↦ ⟦",
-              "      number(φ) ↦ ⟦⟧,",
-              "      bytes(data) ↦ ⟦⟧,",
-              "      string(φ) ↦ ⟦⟧,",
-              "      λ ⤍ Package",
-              "    ⟧,",
-              "    λ ⤍ Package",
-              "  ⟧",
-              "⟧}"
+            [ "{⟦"
+            , "  org ↦ ⟦"
+            , "    eolang ↦ ⟦"
+            , "      number(φ) ↦ ⟦⟧,"
+            , "      bytes(data) ↦ ⟦⟧,"
+            , "      string(φ) ↦ ⟦⟧,"
+            , "      λ ⤍ Package"
+            , "    ⟧,"
+            , "    λ ⤍ Package"
+            , "  ⟧"
+            , "⟧}"
             ]
         ]
 
